@@ -1,12 +1,14 @@
-from enum import IntEnum
-from abc import ABC, abstractmethod
-from mewpy.utils.constants import EAConstants, ModelConstants
-from mewpy.optimization.ea import Solution
-from mewpy.simulation import get_simulator
-from collections import OrderedDict
-import warnings
-import numpy as np
 import copy
+import warnings
+from abc import ABC, abstractmethod
+from collections import OrderedDict
+from enum import IntEnum
+
+import numpy as np
+
+from ..optimization.ea import Solution
+from ..simulation import get_simulator
+from ..util.constants import EAConstants, ModelConstants
 
 
 class Strategy(IntEnum):
@@ -46,7 +48,7 @@ class OUBounder(object):
     def __init__(self, lower_bound, upper_bound):
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
-        self.range = [self.upper_bound[i]-self.lower_bound[i] +
+        self.range = [self.upper_bound[i] - self.lower_bound[i] +
                       1 for i in range(len(self.lower_bound))]
 
     def __call__(self, candidate, args):
@@ -69,13 +71,14 @@ class AbstractProblem(ABC):
     :param kwargs: Additional parameters dictionary
     """
 
-    def __init__(self, model, fevaluation=None,**kwargs):
+    def __init__(self, model, fevaluation=None, **kwargs):
         self.model = model
         self.fevaluation = fevaluation
         self.number_of_objectives = len(self.fevaluation)
-        
+
         # simulation context : defines the simulations environment
-        self.simul_context = None
+        self._reset_solver = kwargs.get('reset_solver', ModelConstants.RESET_SOLVER)
+        self._simul = None
         # The target product reaction id may be specified when optimizing for a single product.
         # Only required for probabilistic modification targeting.
         self.product = kwargs.get('product', None)
@@ -112,7 +115,7 @@ class AbstractProblem(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def translate(self, candidate, reverse=False):
+    def encode(self, candidate):
         """The generator function for the problem."""
         raise NotImplementedError
 
@@ -122,7 +125,7 @@ class AbstractProblem(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def solution_to_constraints(self,solution):
+    def solution_to_constraints(self, solution):
         """Converts a decoded solution to metabolict constraints."""
         raise NotImplementedError
 
@@ -138,15 +141,15 @@ class AbstractProblem(ABC):
 
     @property
     def simulator(self):
-        if self.simul_context is None:
-            self.simul_context = get_simulator(
-                self.model, reference=self._reference)
-            self.simul_context.environmental_conditions = self.environmental_conditions
-            self.simul_context.constraints = self.persistent_constraints
-        return self.simul_context
+        if self._simul is None:
+            self._simul = get_simulator(
+                self.model, envcond=self.environmental_conditions,
+                constraints=self.persistent_constraints,
+                reference=self._reference, reset_solver=self._reset_solver)
+        return self._simul
 
     def reset_simulator(self):
-        self.simul_context = None
+        self._simul = None
 
     def __str__(self):
         if self.number_of_objectives > 1:
@@ -174,7 +177,7 @@ class AbstractProblem(ABC):
         """
         :returns: The constrainst enconded into an individual.
         """
-        return solution_to_constaints(self.decode(solution.candidate))
+        return self.solution_to_constraints(self.decode(solution.candidate))
 
     def get_environmental_conditions(self):
         return self.environmental_conditions
@@ -191,7 +194,7 @@ class AbstractProblem(ABC):
         :returns: A list of fitness.
         """
         p = []
-        decoded ={}
+        decoded = {}
         # decoded constraints
         if decode:
             decoded = self.decode(solution)
@@ -221,7 +224,7 @@ class AbstractProblem(ABC):
     def is_maximization(self):
         return all([f.maximize for f in self.fevaluation])
 
-    def simplify(self, solution, tolerance=1e-6,):
+    def simplify(self, solution, tolerance=1e-6, ):
         """
         Simplify a solution by removing the modification that do not affect the final fitness value.
         Two solutions are considered different if the maximum allowed difference between objective values is exceeded.
@@ -234,7 +237,7 @@ class AbstractProblem(ABC):
 
         """
 
-        values = self.translate(solution.values, reverse=True)
+        values = self.encode(solution.values)
         fitness = self.evaluate_solution(values)
         one_to_remove = {}
         # single removal
@@ -242,7 +245,7 @@ class AbstractProblem(ABC):
             simul_constraints = copy.copy(values)
             simul_constraints.remove(entry)
             fit = self.evaluate_solution(simul_constraints)
-            diff = np.abs(np.array(fit)-np.array(fitness))
+            diff = np.abs(np.array(fit) - np.array(fitness))
             is_equal = False
             if isinstance(tolerance, float):
                 is_equal = np.all(diff <= tolerance)
@@ -257,7 +260,7 @@ class AbstractProblem(ABC):
 
         # test all simultaneous removal
         fit = self.evaluate_solution(simul_constraints)
-        diff = np.abs(np.array(fit)-np.array(fitness))
+        diff = np.abs(np.array(fit) - np.array(fitness))
         is_equal = False
         if isinstance(tolerance, float):
             is_equal = np.all(diff <= tolerance)
@@ -265,8 +268,8 @@ class AbstractProblem(ABC):
             is_equal = np.all(diff <= np.array(tolerance))
 
         if is_equal:
-            v = self.translate(simul_constraints)
-            c = self.decode(simul_constraints)
+            v = self.encode(simul_constraints)
+            c = self.solution_to_constraints(simul_constraints)
             simplification = Solution(v, fitness, c)
             return [simplification]
         else:
@@ -274,8 +277,8 @@ class AbstractProblem(ABC):
             for entry, fit in one_to_remove.items():
                 simul_constraints = copy.copy(values)
                 simul_constraints.remove(entry)
-                v = self.translate(simul_constraints)
-                c = self.decode(simul_constraints)
+                v = self.encode(simul_constraints)
+                c = self.solution_to_constraints(simul_constraints)
                 simplification = Solution(v, fitness, c)
                 res.append(simplification)
             return res
@@ -295,7 +298,6 @@ class AbstractKOProblem(AbstractProblem):
             model, fevaluation=fevaluation, **kwargs)
         self.strategy = Strategy.KO
 
-
     def decode(self, candidate):
         decoded = {}
         for idx in candidate:
@@ -306,6 +308,15 @@ class AbstractKOProblem(AbstractProblem):
                     idx, len(self.target_list[idx])))
         return decoded
 
+    def encode(self, candidate):
+        """
+        Translates a candidate solution in problem specific representation to
+        an iterable of ids, or (ids, folds).
+
+        :param candidate: The candidate representation.
+        """
+        return set([self.target_list.index(k) for k in candidate])
+
     def solution_to_constraints(self, decoded_candidate):
         """
         Converts a candidate, a dictionary of reactions, into a dictionary of constraints
@@ -318,12 +329,10 @@ class AbstractKOProblem(AbstractProblem):
         The KO list index bounder
         """
         if self._bounder is None:
-            max = len(self.target_list)-1
+            max = len(self.target_list) - 1
             self._bounder = KOBounder(0, max)
         return self._bounder
 
-        
-
     def generator(self, random, args):
         """
         Generates a solution, a random int set with length in range min_solution_size to max_solution_size.
@@ -332,24 +341,9 @@ class AbstractKOProblem(AbstractProblem):
         solution_size = random.uniform(
             self.candidate_min_size, self.candidate_max_size)
         while len(solution) < solution_size:
-            solution.add(random.randint(0, len(self.target_list)-1))
+            solution.add(random.randint(0, len(self.target_list) - 1))
         return solution
 
-    def translate(self, candidate, reverse=False):
-        """
-        Translates a candidate solution in problem specific representation to
-        an iterable of ids, or (ids, folds).
-
-        :param candidate: The candidate representation.
-        :param boolean reverse: Performs a reverse translation.
-
-        """
-        if not reverse:
-            return {self.target_list[idx] for idx in candidate}
-        else:
-            return [self.target_list.index(k) for k in candidate]
-
-
 
 class AbstractOUProblem(AbstractProblem):
     """ Base class for Over/Under expression optimization problems
@@ -372,7 +366,6 @@ class AbstractOUProblem(AbstractProblem):
         self.levels = kwargs.get('levels', EAConstants.LEVELS)
         self._reference = kwargs.get('reference', None)
 
-
     def decode(self, candidate):
         """The decoder function for the problem. Needs to be implemented by extending classes."""
         decoded = {}
@@ -380,11 +373,24 @@ class AbstractOUProblem(AbstractProblem):
             try:
                 rxn = self.target_list[idx]
                 lv = self.levels[lv_idx]
-                decoded[rxn]=lv
+                decoded[rxn] = lv
             except IndexError:
-                raise IndexError("Index out of range")    
+                raise IndexError("Index out of range")
         return decoded
 
+    def encode(self, candidate):
+        """
+        Translates a candidate solution in problem specific representation to
+        an iterable of ids, or (ids, folds).
+
+        :param iterable candidate: The candidate representation.
+        :returns: a list of index tupple (modification_target_index,level_index). The indexes are
+                  problem dependent.
+        """
+
+        return set([(self.target_list.index(k), self.levels.index(lv))
+                    for k, lv in candidate.items()])
+
     def solution_to_constraints(self, decoded_candidate):
         """
         Decodes a candidate, a dictionary of reactions, into a dictionary of constraints
@@ -399,8 +405,8 @@ class AbstractOUProblem(AbstractProblem):
         :returns: a OUBounder object.
         """
         if self._bounder is None:
-            max_idx = len(self.target_list)-1
-            max_lv = len(self.levels)-1
+            max_idx = len(self.target_list) - 1
+            max_lv = len(self.levels) - 1
             self._bounder = OUBounder([0, 0], [max_idx, max_lv])
         return self._bounder
 
@@ -416,8 +422,8 @@ class AbstractOUProblem(AbstractProblem):
         solution_size = random.uniform(
             self.candidate_min_size, self.candidate_max_size)
         while len(solution) < solution_size:
-            idx = random.randint(0, len(self.target_list)-1)
-            lv = random.randint(0, len(self.levels)-1)
+            idx = random.randint(0, len(self.target_list) - 1)
+            lv = random.randint(0, len(self.levels) - 1)
             solution.add((idx, lv))
         return solution
 
@@ -440,9 +446,9 @@ class AbstractOUProblem(AbstractProblem):
         """
         if level > 1:
             if wt >= 0:
-                return (level*wt, ModelConstants.REACTION_UPPER_BOUND)
+                return (level * wt, ModelConstants.REACTION_UPPER_BOUND)
             else:
-                return (-1*ModelConstants.REACTION_UPPER_BOUND, level*wt)
+                return (-1 * ModelConstants.REACTION_UPPER_BOUND, level * wt)
         else:
             return (0, level * wt) if wt >= 0 else (level * wt, 0)
 
@@ -479,23 +485,3 @@ class AbstractOUProblem(AbstractProblem):
             constraints[ko_rxn] = (0, 0)
             constraints[ou_rxn] = self.ou_constraint(lv, fwt)
         return constraints
-
-    def translate(self, candidate, reverse=False):
-        """
-        Translates a candidate solution in problem specific representation to
-        an iterable of ids, or (ids, folds).
-
-        :param iterable candidate: The candidate representation.
-        :param boolean reverse: Performs the reverse translation.
-        :returns: If reverse, a list of index tupple (modification_target_index,level_index). The indexes are
-                  problem dependent.
-                  If not reverse, a dictionary of {modification_target: level}
-        """
-        if not reverse:
-            return {self.target_list[idx]: self.levels[lv_idx]
-                    for idx, lv_idx in candidate}
-        else:
-            return [(self.target_list.index(k), self.levels.index(lv))
-                    for k, lv in candidate.items()]
-
-   
\ No newline at end of file
