@@ -632,7 +632,7 @@ class Simulation(CBModelContainer, Simulator):
 
         result = SimulationResult(self.model, solution.fobj, fluxes=solution.values, status=status,
                                   envcond=self.environmental_conditions, model_constraints=self.constraints,
-                                  simul_constraints=constraints, maximize=maximize)
+                                  simul_constraints=constraints, maximize=maximize, method=method)
         return result
 
     def FVA(self, obj_frac=0.9, reactions=None, constraints=None, loopless=False, internal=None, solver=None,
